import { INotebookTracker, } from '@jupyterlab/notebook';
import { ICodeMirror } from '@jupyterlab/codemirror';
import { CodeCell } from '@jupyterlab/cells';
import { setupKatScriptCodeMirror } from './codemirror-katscript-python';
const plugin = {
    id: 'katscript-jupyter',
    autoStart: true,
    requires: [INotebookTracker, ICodeMirror],
    activate,
};
function activate(app, tracker, codemirror) {
    setupKatScriptCodeMirror(codemirror);
    const style = document.createElement('style');
    style.type = 'text/css';
    style.innerHTML =
        '.katscript-highlight-background .katscript { background-color: rgba(0, 0, 0, 0.04); }';
    document.getElementsByTagName('head')[0].appendChild(style);
    tracker.widgetAdded.connect(activate_katscript);
}
function check_all(sender, mimeType) {
    sender.content.widgets.forEach(cell => {
        if (cell instanceof CodeCell) {
            cell.model.mimeType = mimeType;
        }
    });
}
function check_cell(sender, cell, mimeType) {
    if (cell instanceof CodeCell) {
        cell.model.mimeType = mimeType;
    }
}
function activate_katscript(sender, panel) {
    panel.sessionContext.ready.then(() => {
        if (panel.model.defaultKernelLanguage === 'python') {
            check_all(panel, 'text/x-katscript-python');
            panel.content.activeCellChanged.connect((...args) => {
                check_cell(...args, 'text/x-katscript-python');
            });
            panel.content.addClass('katscript-highlight-background');
        }
    });
}
export default plugin;
